@@ -1,8 +1,8 @@
 from django_tables2 import SingleTableMixin
 from django.shortcuts import redirect
 from django.http import HttpResponseRedirect
-from django.urls import reverse_lazy
-from django.db.models import Q
+from django.urls import reverse_lazy, reverse
+from django.db.models import Q, ProtectedError
 from pandas import isna, DataFrame as DF, to_datetime
 import inspect
 from django.contrib import messages
@@ -12,6 +12,46 @@ from django_pandas.io import read_frame
 from django.contrib.auth.mixins import UserPassesTestMixin
 import plotly.express as px
 from plotly import offline
+from django.views.generic import DeleteView
+
+
+class DeleteProtectedView(DeleteView):
+    ''' Subclass of django.views.genderic.DeleteView that handles 
+    the attempted deltion of Protected FKs '''
+    protected_error_msg = "This employee is associated with data and cannot be deleted"
+
+    def get_onclick_cancel_action(self, href=None):
+        if href == None:
+            href = self.request.session['next']
+        if href:
+            return f"location.href = '{href}'"
+        else:
+            return "history.back()"
+
+    def get_protected_error_msg(self):
+        ''' Returns the error message to be displayed  '''
+        obj = self.get_object()
+        return f"{obj} is associted with exisiting data and cannot be deleted"
+
+
+    def post(self, request, *args, **kwargs):
+        #Try super but catch ProtectedError
+        try:
+            return super().post(request, *args, **kwargs)
+        except ProtectedError:
+            #When emp FKs to PartRun onDelete is set to PROTECT
+            messages.error(request, self.get_protected_error_msg())
+            http_referer = request.META.get('HTTP_REFERER')
+            if http_referer:
+                return HttpResponseRedirect(http_referer)
+            else:
+                return self.get_success_url()
+
+    def get_context_data(self, **kwargs):
+        context = super().get_context_data(**kwargs)
+        context['onclick_cancel_action'] = self.get_onclick_cancel_action()
+        return context
+
 
 class SaveFilterMixinNT:
     """ This Mixin Can be used to save
@@ -50,19 +90,52 @@ class InlineFormsetMixin:
     factories = [] # list of dictionaries that must contain the key factory and the value of a formset factory instance, helper and herder are optional
     form_helper = None
     template_name = 'django_aux/inline-formset.html'
-    addlines_url = None # set this in the view to have the addlines btn redirect here with self.object.pk as an arg
-    
-    def get_context_data(self, *args, factories=None, **kwargs):
-        context = super().get_context_data(*args, **kwargs)
-        if factories == None:
-            factories = [fd.copy() for fd in self.factories]
-            for fd in factories:
-                if self.request.POST:
-                    fd['factory'] = fd['factory'](self.request.POST, instance=self.object)
-                else:
-                    fd['factory'] = fd['factory'](instance=self.object)
+
+    def get_current_extra(self):
+        extra = self.request.GET.get('extra')
+        if not extra:
+            return 1
+        extra = int(extra)
+        if extra > 10:
+            return 10
+        else:
+            return extra 
+
+    def set_extra_on_factories(self, context):
+        if self.request.method != 'GET':
+            return
+        for fd in context['factories']:
+            fd.get('factory').extra = int(self.get_current_extra())
+
+    def add_factories_to_context(self, context):
+        factories = [fd.copy() for fd in self.factories]
+        for fd in factories:
+            if self.request.POST:
+                fd['factory'] = fd['factory'](self.request.POST, instance=self.object)
+            else:
+                fd['factory'] = fd['factory'](instance=self.object)
         context['factories'] = factories
-        context['form_helper']=self.form_helper if self.form_helper else None
+
+    def add_addlines_url_to_context(self, context):
+        add_extra = self.get_current_extra() + 1
+        context['addlines_url'] = self.request.path_info + f"?extra={add_extra}"
+
+    def add_removelines_url_to_context(self, context):
+        remove_extra = self.get_current_extra() - 1
+        if remove_extra <=0:
+            remove_extra = 1
+        context['removelines_url'] = self.request.path_info + f"?extra={remove_extra}"
+
+    def get_context_data(self, *args, **kwargs):
+        context = super().get_context_data(*args, **kwargs)
+        self.add_factories_to_context(context)
+        self.set_extra_on_factories(context)
+        self.add_addlines_url_to_context(context)
+        self.add_removelines_url_to_context(context)
+        context['form_helper'] = self.form_helper if self.form_helper else None
+        context['add_addlines_btn'] = True
+        context['add_removelines_btn'] = True
+        context['add_cancel_btn'] = True
         return context
             
     def post(self, request, *args, **kwargs):
@@ -73,8 +146,7 @@ class InlineFormsetMixin:
         context = self.get_context_data()
         form = self.get_form()
         if not form.is_valid():
-            print(form.errors)
-            return self.form_invalid(form)
+            return self.form_invalid(form, factories=context['factories'])
         fdsv = True
         for fd in context['factories']:
             if not fd['factory'].is_valid():
@@ -97,16 +169,6 @@ class InlineFormsetMixin:
     def form_invalid(self, form, factories):
         return self.render_to_response(self.get_context_data(form=form, factories=factories))
 
-    def get_success_url(self):
-        if 'addlines' in self.request.POST and self.addlines_url:
-            #user defined a redirect url for the addlines btn, default behavior sends pk as arg
-            return reverse_lazy(self.addlines_url, kwargs={'pk':self.object.pk})
-        elif 'addlines' in self.request.POST and not self.addlines_url:
-            #adding lines to form but no redirect url given. Use default behavior
-            return self.request.path_info
-        else:
-            #add lines was not clicked, use default success_url
-            return super().get_success_url()
 
 class DashFilterView(FilterView):
     ''' A view for passing filter qs to a dash app '''
@@ -169,6 +231,9 @@ class SaveFormMixin:
 class RedirectPrevMixin:
     ''' This mixin will redirect user to the page they came from if 
     form successful OR if "cancel" is in post data  (Uses session data)'''
+    redirect_exceptions = [] # list of paths or partial paths that should not be redirected to and the name of the view to use instead
+    # i.e.
+    # redirect_exceptions = [('person-delete', 'person-lookup')]
 
     @property
     def form_takes_request_arg(self):
@@ -189,14 +254,24 @@ class RedirectPrevMixin:
         kwargs['request'] = self.request
         return kwargs
 
+    def get_next_is_exception(self, next):
+        for pattern, reverse_name  in self.redirect_exceptions:
+            if pattern in next:
+                return reverse_name  
+        return None
+
     def get(self, request, *args, **kwargs):
         ''' Extends the get method to store where the user was prior to this page '''
-        next = self.request.META.get('HTTP_REFERER')
+        next = request.META.get('HTTP_REFERER')
         if next == None: 
             next = ''
-        mask = request.path in next # redirected from the same page, dont overrwrite next
-        if not mask:
-            request.session['next'] = self.request.META.get('HTTP_REFERER')
+        same_path = request.path in next # redirected from the same page, dont overrwrite next
+        exception_reverse_name = self.get_next_is_exception(next)
+        if exception_reverse_name:
+            request.session['next'] = reverse(exception_reverse_name)
+        elif not same_path:
+            request.session['next'] = next
+
         return super().get(request, *args, **kwargs)
 
     def get_success_url(self):
